from __future__ import absolute_import
import datetime
import json

import six


DATE_FMT = "%Y-%m-%d"
ISO8601_FMT = "%Y-%m-%dT%H:%M:%SZ"
JAVASCRIPT_FMT = "datetime.datetime(%Y, %m, %d, %H, %M, %S)"


try:
    # Python 3
    from datetime import timezone

    UTC = timezone.utc
except ImportError:
    # Python 2
    class UTC(datetime.tzinfo):
        def utcoffset(self, dt):
            return datetime.timedelta(0)

        def tzname(self, dt):
            return "UTC"

        def dst(self, dt):
            return datetime.timedelta(0)

    UTC = UTC()


def _datetime_encoder(obj):
    if isinstance(obj, datetime.datetime):
        try:
            return obj.strftime(ISO8601_FMT)
        except ValueError:
            # Prior to 1900 in Py2
            return "%sZ" % obj.isoformat().split("+")[0]
    elif isinstance(obj, datetime.date):
        try:
            return obj.strftime(DATE_FMT)
        except ValueError:
            # Prior to 1900 in Py2
            return "%s" % obj.isoformat().split("T")[0]

    raise TypeError


def _datetime_decoder(dict_):
    for key, value in six.iteritems(dict_):
        # The built-in `json` library will `unicode` strings, except for empty
        # strings which are of type `str`. `jsondate` patches this for
        # consistency so that `unicode` is always returned.
        if value == "":
            dict_[key] = six.text_type()
            continue

        try:
            dict_[key] = datetime.datetime.strptime(
                value, ISO8601_FMT
            ).replace(tzinfo=UTC)
        except (ValueError, TypeError):
            try:
                date_obj = datetime.datetime.strptime(value, DATE_FMT)
                dict_[key] = date_obj.date()
            except (ValueError, TypeError):
                try:
                    datetime_obj = datetime.datetime.strptime(
                        value, JAVASCRIPT_FMT
                    )
                    dict_[key] = datetime_obj
                except (ValueError, TypeError):
                    continue

    return dict_


def dumps(*args, **kwargs):
    kwargs["default"] = _datetime_encoder
    return json.dumps(*args, **kwargs)


def dump(*args, **kwargs):
    kwargs["default"] = _datetime_encoder
    return json.dump(*args, **kwargs)


def loads(*args, **kwargs):
    kwargs["object_hook"] = _datetime_decoder
    return json.loads(*args, **kwargs)


def load(*args, **kwargs):
    kwargs["object_hook"] = _datetime_decoder
    return json.load(*args, **kwargs)



from __future__ import absolute_import
import datetime
import json
import unittest

import jsondate3
import six

StringIO = six.moves.cStringIO


class JSONDateTests(unittest.TestCase):
    def assertTypeAndValue(self, expected_type, expected_value, result):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(expected_value, result)

    def test_dumps_empty_roundtrips(self):
        self.assertEqual({}, jsondate3.loads(jsondate3.dumps({})))

    def test_dumps_str_roundtrips(self):
        # Generates a ValueError from _datetime_object_hook
        orig_dict = dict(foo="bar")
        self.assertEqual(
            orig_dict, jsondate3.loads(jsondate3.dumps(orig_dict))
        )

    def test_dump_unicode_roundtrips(self):
        orig_dict = {u"foo": u"bar", "empty": u""}
        # json module broken: unicode objects, empty-string objects are str
        result = json.loads(json.dumps(orig_dict))
        self.assertTypeAndValue(six.text_type, u"bar", result[u"foo"])
        self.assertTypeAndValue(six.text_type, "", result[u"empty"])

        # jsondate fix: always return unicode objects
        result = jsondate3.loads(jsondate3.dumps(orig_dict))
        self.assertTypeAndValue(six.text_type, u"bar", result[u"foo"])
        self.assertTypeAndValue(six.text_type, u"", result[u"empty"])

    def test_dumps_none_roundtrips(self):
        # Generates a TypeError from _datetime_object_hook
        orig_dict = dict(foo=None)
        self.assertEqual(
            orig_dict, jsondate3.loads(jsondate3.dumps(orig_dict))
        )

    def test_dumps_datetime_roundtrips(self):
        orig_dict = dict(
            created_at=datetime.datetime(2011, 1, 1, tzinfo=jsondate3.UTC)
        )
        new_dict = jsondate3.loads(jsondate3.dumps(orig_dict))
        self.assertEqual(orig_dict, new_dict)

    def test_dumps_date_roundtrips(self):
        orig_dict = dict(created_at=datetime.date(2011, 1, 1))
        self.assertEqual(
            orig_dict, jsondate3.loads(jsondate3.dumps(orig_dict))
        )

    def test_dumps_old_datetime_roundtrip(self):
        orig_dict = dict(
            created_at=datetime.datetime(1899, 1, 1, tzinfo=jsondate3.UTC)
        )
        new_dict = jsondate3.loads(jsondate3.dumps(orig_dict))
        self.assertEqual(orig_dict, new_dict)

    def test_dumps_old_date_roundtrip(self):
        orig_dict = dict(created_at=datetime.date(1899, 1, 1))
        self.assertEqual(
            orig_dict, jsondate3.loads(jsondate3.dumps(orig_dict))
        )

    def test_dumps_datelike_string_does_not_roundtrip(self):
        """A string that looks like a date *will* be interpreted as a date.

        If for whatever reason, you don't want that to happen, yo'll need to
        do some pre or post-processing to fixup the results.
        """
        orig_dict = dict(created_at="2011-01-01")
        expected = dict(created_at=datetime.date(2011, 1, 1))
        self.assertEqual(expected, jsondate3.loads(jsondate3.dumps(orig_dict)))

    def test_dump_datetime_roundtrips(self):
        orig_dict = dict(created_at=datetime.date(2011, 1, 1))
        fileobj = StringIO()
        jsondate3.dump(orig_dict, fileobj)
        fileobj.seek(0)
        self.assertEqual(orig_dict, jsondate3.load(fileobj))

    def test_unexpected_type_raises(self):
        dict_ = {"foo": set(["a"])}
        with self.assertRaises(TypeError):
            jsondate3.dumps(dict_)



import codecs
import os
import setuptools

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


setuptools.setup(
    name="jsondate3-aware",
    version="0.1.4",
    url="https://github.com/freelawproject/jsondate3-aware",
    license="BSD",
    author="Rick Harris, Maciej nitZ Krol, Free Law Project",
    author_email="rconradharris@gmail.com, nitz@o2.pl, info@free.law",
    description="JSON with tz-aware datetime support",
    long_description=read("README.rst"),
    packages=setuptools.find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms="any",
    install_requires=["six", "iso8601"],
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
